@@ -1,6 +1,6 @@
 var searchData=
 [
-  ['begin_3',['begin',['../a00018.html#a58e73443eadd3f933653844d74bed783',1,'RNifti::NiftiImageData::begin() const'],['../a00018.html#acd4e31b5a4daf38193831967ae93c20b',1,'RNifti::NiftiImageData::begin()']]],
+  ['begin_3',['begin',['../a00018.html#a58e73443eadd3f933653844d74bed783',1,'RNifti::NiftiImageData::begin() const'],['../a00018.html#acd4e31b5a4daf38193831967ae93c20b',1,'RNifti::NiftiImageData::begin()'],['../a00050.html#ad680080c8c979dea5fad9d2364249cf3',1,'RNifti::SquareMatrix::begin() const'],['../a00050.html#a411c46aca8ccc1e0d5e36f87c798ea86',1,'RNifti::SquareMatrix::begin()']]],
   ['blob_4',['blob',['../a00018.html#a09885563011e9a614fa35aeaa29bf72d',1,'RNifti::NiftiImageData']]],
   ['block_5',['Block',['../a00058.html',1,'RNifti::NiftiImage::Block'],['../a00054.html#a863694f20c787839093e13f41af70255',1,'RNifti::NiftiImage::block(const int i) const'],['../a00054.html#a96a7386170f4146a834cbefe280b35be',1,'RNifti::NiftiImage::block(const int i)'],['../a00058.html#aa5219a8cf709eddba6848896664fe07c',1,'RNifti::NiftiImage::Block::Block()']]],
   ['bytesperpixel_6',['bytesPerPixel',['../a00018.html#a1c00c50a2cd16c2eab8435fbf05c6b94',1,'RNifti::NiftiImageData']]]
